package com.alicloud.openservices.tablestore.core.protocol;

import java.util.HashMap;
import java.util.Map;

import com.alicloud.openservices.tablestore.ClientException;
import com.alicloud.openservices.tablestore.model.search.agg.AggregationResult;
import com.alicloud.openservices.tablestore.model.search.agg.AggregationResults;
import com.alicloud.openservices.tablestore.model.search.agg.AvgAggregationResult;
import com.alicloud.openservices.tablestore.model.search.agg.CountAggregationResult;
import com.alicloud.openservices.tablestore.model.search.agg.DistinctCountAggregationResult;
import com.alicloud.openservices.tablestore.model.search.agg.MaxAggregationResult;
import com.alicloud.openservices.tablestore.model.search.agg.MinAggregationResult;
import com.alicloud.openservices.tablestore.model.search.agg.SumAggregationResult;
import com.google.protobuf.ByteString;
import com.google.protobuf.InvalidProtocolBufferException;

class SearchAggregationResultBuilder {

    private static AvgAggregationResult buildAvgAggregationResult(String aggName, ByteString aggBody)
        throws InvalidProtocolBufferException {
        Search.AvgAggregationResult aggResult = Search.AvgAggregationResult.parseFrom(aggBody);
        AvgAggregationResult result = new AvgAggregationResult();
        result.setAggName(aggName);
        result.setValue(aggResult.getValue());
        return result;
    }

    private static MaxAggregationResult buildMaxAggregationResult(String aggName, ByteString aggBody)
        throws InvalidProtocolBufferException {
        Search.MaxAggregationResult aggResult = Search.MaxAggregationResult.parseFrom(aggBody);
        MaxAggregationResult result = new MaxAggregationResult();
        result.setAggName(aggName);
        result.setValue(aggResult.getValue());
        return result;
    }

    private static MinAggregationResult buildMinAggregationResult(String aggName, ByteString aggBody)
        throws InvalidProtocolBufferException {
        Search.MinAggregationResult aggResult = Search.MinAggregationResult.parseFrom(aggBody);
        MinAggregationResult result = new MinAggregationResult();
        result.setAggName(aggName);
        result.setValue(aggResult.getValue());
        return result;
    }

    private static SumAggregationResult buildSumAggregationResult(String aggName, ByteString aggBody)
        throws InvalidProtocolBufferException {
        Search.SumAggregationResult aggResult = Search.SumAggregationResult.parseFrom(aggBody);
        SumAggregationResult result = new SumAggregationResult();
        result.setAggName(aggName);
        result.setValue(aggResult.getValue());
        return result;
    }

    private static DistinctCountAggregationResult buildDistinctCountAggregationResult(String aggName,
        ByteString aggBody)
        throws InvalidProtocolBufferException {
        Search.DistinctCountAggregationResult aggResult = Search.DistinctCountAggregationResult.parseFrom(aggBody);
        DistinctCountAggregationResult result = new DistinctCountAggregationResult();
        result.setAggName(aggName);
        result.setValue(aggResult.getValue());
        return result;
    }

    private static CountAggregationResult buildCountAggregationResult(String aggName, ByteString aggBody)
        throws InvalidProtocolBufferException {
        Search.CountAggregationResult aggResult = Search.CountAggregationResult.parseFrom(aggBody);
        CountAggregationResult result = new CountAggregationResult();
        result.setAggName(aggName);
        result.setValue(aggResult.getValue());
        return result;
    }

    private static AggregationResult buildAggResult(Search.AggregationResult aggregationResult)
        throws InvalidProtocolBufferException {
        switch (aggregationResult.getType()) {
            case AGG_AVG:
                return buildAvgAggregationResult(aggregationResult.getName(), aggregationResult.getAggResult());
            case AGG_MAX:
                return buildMaxAggregationResult(aggregationResult.getName(), aggregationResult.getAggResult());
            case AGG_MIN:
                return buildMinAggregationResult(aggregationResult.getName(), aggregationResult.getAggResult());
            case AGG_SUM:
                return buildSumAggregationResult(aggregationResult.getName(), aggregationResult.getAggResult());
            case AGG_DISTINCT_COUNT:
                return buildDistinctCountAggregationResult(aggregationResult.getName(),
                    aggregationResult.getAggResult());
            case AGG_COUNT:
                return buildCountAggregationResult(aggregationResult.getName(), aggregationResult.getAggResult());
            default:
                throw new ClientException("unsupported aggType: " + aggregationResult.getType());
        }
    }

    static AggregationResults buildAggregationResults(Search.AggregationsResult aggregationsResult)
        throws InvalidProtocolBufferException {
        AggregationResults aggregationResults = new AggregationResults();
        Map<String, AggregationResult> map = new HashMap<String, AggregationResult>();

        for (Search.AggregationResult s : aggregationsResult.getAggResultsList()) {
            map.put(s.getName(), buildAggResult(s));
        }
        aggregationResults.setResultMap(map);
        return aggregationResults;
    }

    static AggregationResults buildAggregationResultsFromByteString(ByteString agg)
        throws InvalidProtocolBufferException {
        Search.AggregationsResult aggregationsResult = Search.AggregationsResult.parseFrom(agg);
        return buildAggregationResults(aggregationsResult);
    }

}
