package com.alicloud.openservices.tablestore.model;

import com.alicloud.openservices.tablestore.core.utils.Preconditions;

import javax.xml.crypto.Data;
import java.nio.ByteBuffer;
import java.util.List;

public class BulkExportResponse extends Response {
    private ConsumedCapacity consumedCapacity;

    private ByteBuffer rows;

    private PrimaryKey nextStartPrimaryKey;

    private byte[] nextToken;

    private DataBlockType dataBlockType = DataBlockType.DBT_PLAIN_BUFFER;

    public BulkExportResponse(Response meta, ConsumedCapacity consumedCapacity) {
        super(meta);
        Preconditions.checkNotNull(consumedCapacity);
        this.consumedCapacity = consumedCapacity;
    }

    public void setRows(ByteBuffer rows) {
        this.rows = rows;
    }

    public void setNextStartPrimaryKey(PrimaryKey nextStartPrimaryKey) {
        this.nextStartPrimaryKey = nextStartPrimaryKey;
    }

    public void setDataBlockType(DataBlockType dataBlockType) {
        this.dataBlockType = dataBlockType;
    }

    public DataBlockType getDataBlockType() {
        return dataBlockType;
    }

    /**
     * 获取此次操作消耗的CapacityUnit。
     *
     * @return 此次操作消耗的CapacityUnit。
     */
    public ConsumedCapacity getConsumedCapacity() {
        return consumedCapacity;
    }

    /**
     * 获取本次查询返回的所有行。
     *
     * @return 所有行
     */
    public ByteBuffer getRows() {
        return rows;
    }

    /**
     * 获取下一次查询的范围的起始边界。
     * 若为null，则代表本次查询已经返回所有数据，无需再次查询。
     *
     * @return 若该范围内的数据还未读取完毕，则返回下一行的主键，否则返回null
     */
    public PrimaryKey getNextStartPrimaryKey() {
        return nextStartPrimaryKey;
    }

    public byte[] getNextToken() {
        return nextToken;
    }

    public void setNextToken(byte[] nextToken) {
        this.nextToken = nextToken;
    }

    public boolean hasNextToken() {
        return (nextToken != null) && (nextToken.length > 0);
    }
}