package com.alicloud.openservices.tablestore.model.search.agg;

import com.alicloud.openservices.tablestore.core.protocol.SearchAggregationBuilder;
import com.alicloud.openservices.tablestore.core.utils.ValueUtil;
import com.alicloud.openservices.tablestore.model.ColumnValue;
import com.google.protobuf.ByteString;

/**
 * 根据某一个字段统计去重后的文档总数。 该总数为大约值，数据量特别大时候存在一定的误差。
 */
public class DistinctCountAggregation implements Aggregation {

    /**
     * 聚合的名字，之后从聚合结果列表中根据该名字拿到聚合结果
     */
    private String aggName;
    /**
     * 字段名字
     */
    private String fieldName;

    /**
     * 缺失字段的默认值。
     * <p>如果一个文档缺少该字段，则采用什么默认值</p>
     */
    private ColumnValue missing;

    public DistinctCountAggregation() {
    }

    private DistinctCountAggregation(Builder builder) {
        aggName = builder.aggName;
        fieldName = builder.fieldName;
        missing = builder.missing;
    }
    protected static Builder newBuilder() {
        return new Builder();
    }

    @Override
    public String getAggName() {
        return aggName;
    }

    @Override
    public AggregationType getAggType() {
        return AggregationType.AGG_DISTINCT_COUNT;
    }

    @Override
    public ByteString serialize() {
        return SearchAggregationBuilder.buildDistinctCountAggregation(this).toByteString();
    }

    public String getFieldName() {
        return fieldName;
    }

    public ColumnValue getMissing() {
        return missing;
    }

    public DistinctCountAggregation setAggName(String aggName) {
        this.aggName = aggName;
        return this;
    }

    public DistinctCountAggregation setFieldName(String fieldName) {
        this.fieldName = fieldName;
        return this;
    }

    public DistinctCountAggregation setMissing(ColumnValue missing) {
        this.missing = missing;
        return this;
    }


    public static final class Builder implements AggregationBuilder {
        private String aggName;
        private String fieldName;
        private ColumnValue missing;

        private Builder() {}

        public Builder aggName(String aggName) {
            this.aggName = aggName;
            return this;
        }

        public Builder fieldName(String fieldName) {
            this.fieldName = fieldName;
            return this;
        }
        /**
         * 缺失字段的默认值。
         * <p>如果一个文档缺少该字段，则采用什么默认值</p>
         */
        public Builder missing(Object missing) {
            this.missing = ValueUtil.toColumnValue(missing);
            return this;
        }

        @Override
        public DistinctCountAggregation build() {
            return new DistinctCountAggregation(this);
        }
    }
}
